{% extends "lava_scheduler_app/job_sidebar.html" %}

{% block content %}
<h2>Annotate Job Failure - {{ job.id }} </h2>

{% if form.errors %}
<h3>Errors found in submission</h3>
{{ form.errors }}
{% endif %}

<form action="{% url lava.scheduler.job.annotate_failure job.pk %}" method="post">
{% csrf_token %}
{{ form }}
<input type="submit" value="Submit" />
</form>
{% endblock %}
