{% extends "lava_scheduler_app/_content.html" %}

{% load django_tables2 %}

{% block content %}

<h2>Device Type Overview</h2>
<p>{{device_status}} devices online</p>
<p>{{health_check_status}} health check jobs passed in 24 hours</p>

{% render_table device_type_table %}

<p><a href="{% url lava.scheduler.alldevices %}">All Devices</a></p>

<p><a href="{% url lava.scheduler.labhealth %}">All Devices Health</a></p>

<h2>Active Jobs</h2>

{% render_table active_jobs_table %}

<a href="{% url lava.scheduler.job.list %}">All jobs</a>

{% endblock %}
